#!/bin/bash
#Automatically Created by slkbuild 0.5.1
#Maintainer: Thorsten Muehlfelder <thenktor@gmx.de>
#url: http://spkg.megous.com/index.php

######Begin Redundant Code######################################
check_for_root() {
	if [ "$UID" != "0" ]; then
		echo "You need to be root"
		exit 1
	fi
}

clean_dirs () {
        for COMPLETED in src pkg; do
                if [ -e $COMPLETED ]; then
                        rm -rf $COMPLETED
                fi
        done
}

clean_old_builds () {
	rm -rf $package.{t[xlgb]z,md5}
	clean_dirs
}

set_pre_permissions() {
	cd $startdir/src
	find . -perm 664 -exec chmod 644 {} \;
	find . -perm 600 -exec chmod 644 {} \;
	find . -perm 444 -exec chmod 644 {} \;
	find . -perm 400 -exec chmod 644 {} \;
	find . -perm 440 -exec chmod 644 {} \;
	find . -perm 777 -exec chmod 755 {} \;
	find . -perm 775 -exec chmod 755 {} \;
	find . -perm 511 -exec chmod 755 {} \;
	find . -perm 711 -exec chmod 755 {} \;
	find . -perm 555 -exec chmod 755 {} \;
}


gzip_man_and_info_pages() {
	for DOCS in man info; do
		if [ -d "$startdir/pkg/usr/share/$DOCS" ]; then
			mv $startdir/pkg/usr/share/$DOCS $startdir/pkg/usr/$DOCS
			if [[ ! "$(ls $startdir/pkg/usr/share)" ]]; then
				rm -rf $startdir/pkg/usr/share
			fi
		fi
		if [ -d "$startdir/pkg/usr/$DOCS" ]; then
			# I've never seen symlinks in info pages....
			if [ "$DOCS" == "man" ]; then
				(cd $startdir/pkg/usr/$DOCS
				for manpagedir in $(find . -type d -name "man*" 2> /dev/null) ; do
					( cd $manpagedir
					for eachpage in $( find . -type l -maxdepth 1 2> /dev/null) ; do
						ln -s $( readlink $eachpage ).gz $eachpage.gz
						rm $eachpage
					done )
				done)
			fi
			find $startdir/pkg/usr/$DOCS -type f -exec gzip -9 '{}' \;
		fi
	done
	[ -a $startdir/pkg/usr/info/dir.gz ] && rm -f $startdir/pkg/usr/info/dir.gz
}

set_post_permissions() {
	for DIRS in usr/share/icons usr/doc; do
		if [ -d "$startdir/pkg/$DIRS" ]; then
			if [ "$DIRS" == "usr/doc" ]; then
				find $startdir/pkg/$DIRS -type f -exec chmod 644 {} \;
				find $startdir/pkg/$DIRS -type d -exec chmod 755 {} \;
			fi
		fi
		[ -d $startdir/pkg/$DIRS ] && chown root:root -R $startdir/pkg/$DIRS
	done
	[ -d $startdir/pkg/usr/bin ] && find $startdir/pkg/usr/bin -user root -group bin -exec chown root:root {} \;
}

copy_build_script() {
	mkdir -p $startdir/pkg/usr/src/$pkgname-$pkgver/
	cp $startdir/build-$pkgname.sh $startdir/pkg/usr/src/$pkgname-$pkgver/build-$pkgname.sh
	[ -f $startdir/SLKBUILD ] && cp $startdir/SLKBUILD	$startdir/pkg/usr/src/$pkgname-$pkgver/SLKBUILD
}

create_package() {
	ls -lR $startdir/pkg
	cd $startdir/pkg
	/sbin/makepkg -l y -c n $startdir/$package.tgz
	cd $startdir
	md5sum $package.tgz > $startdir/$package.md5
}

strip_binaries() {
	cd $startdir/pkg
	find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | \
	xargs strip --strip-unneeded 2> /dev/null
	find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | \
	xargs strip --strip-unneeded 2> /dev/null
}
#########End Redundant Code#####################################
#########Begin Non Redundant Code##############################

prepare_directory() {
	mkdir $startdir/src
	mkdir -p $startdir/pkg/usr/src/$pkgname-$pkgver
	for SOURCES in ${source[@]}; do
		protocol=$(echo $SOURCES | sed 's|:.*||')
	        file=$(basename $SOURCES | awk -F= '{print $NF}')
		if [ ! -f "$file" ]; then
			if [ "$protocol" = "http" -o "$protocol" = "https" -o "$protocol" = "ftp" ]; then
				echo -e "\nDownloading $(basename $SOURCES)\n"
        	                wget $SOURCES -O $file
				if [ ! "$?" == "0" ]; then
					echo "Download failed"
					exit 2
				fi 
			else
				echo "$SOURCES does not appear to be a url nor is it in the directory"
				exit 2
			fi
		fi
		cp -R $file $startdir/src
		if ! [ "$protocol" = "http" -o "$protocol" = "https" -o "$protocol" = "ftp" ]; then
			cp -R $startdir/$(basename $SOURCES) $startdir/pkg/usr/src/$pkgname-$pkgver/
		fi
	done
}

extract_source() {
	cd $startdir/src
	if [[ "$(ls $startdir/src)" ]]; then	
		for FILES in ${source[@]}; do
	        	FILES="$(basename $FILES | awk -F= '{print $NF}')"
			file_type=$(file -biz "$FILES")
			unset cmd
			case "$file_type" in
				*application/x-tar*)
					cmd="tar -xf" ;;
				*application/x-zip*)
					cmd="unzip" ;;
				*application/zip*)
					cmd="unzip" ;;
				*application/x-gzip*)
					cmd="gunzip -d -f" ;;
				*application/x-bzip*)
					cmd="bunzip2 -f" ;;
				*application/x-xz*)
					cmd="xz -d -f" ;;
				*application/x-lzma*)
					cmd="lzma -d -f" ;;
			esac
			if [ "$cmd" != "" ]; then
				echo "$cmd $FILES"
	        	        $cmd $FILES
			fi
		done
	elif [ ! "$source" ]; then
		echo -n "" # lame fix
	else
		echo "no files in the src directory $startdir/src"
		exit 2
	fi
}

build() {
	#
	### spkg section
	#

	cd $startdir/src/spkg-${pkgver}
	patch -p1 < ../01-txz.patch || return 1
	patch -p1 < ../02-size.patch || return 1
	./configure --prefix=/usr  --libdir=/usr/lib${LIBDIRSUFFIX} --mandir=/usr/man || return 1
	make || return 1
	make DESTDIR=$startdir/pkg install

	# prepare directories for lib preload
	mkdir -p $startdir/pkg/usr/lib/spkg/bin
	mkdir -p $startdir/pkg/usr/lib/spkg/preload

	# copy preloaded libs to the package
	cd $startdir/pkg/usr/bin/
	cp `ldd spkg | grep libpopt | awk '{print $3}'` $startdir/pkg/usr/lib/spkg/preload || return 1
	cp `ldd spkg | grep libglib | awk '{print $3}'` $startdir/pkg/usr/lib/spkg/preload || return 1
	cp `ldd spkg | grep libJudy | awk '{print $3}'` $startdir/pkg/usr/lib/spkg/preload || return 1
	cp `ldd spkg | grep libz | awk '{print $3}'` $startdir/pkg/usr/lib/spkg/preload || return 1

	# move spkg to new dir
	mv $startdir/pkg/usr/bin/spkg $startdir/pkg/usr/lib/spkg/bin/ || return 1

	# preload wrapper script
	mkdir -p $startdir/pkg/sbin
	cat << "EOF" > $startdir/pkg/sbin/spkg || return 1
#!/bin/sh
LD_LIBRARY_PATH=/usr/lib/spkg/preload /usr/lib/spkg/bin/spkg "$@"
exit $?
EOF

	# pkgtools wrapper scripts
	cat << "EOF" > $startdir/pkg/sbin/installpkg
#!/bin/sh
/sbin/spkg -i "$@"
exit $?
EOF

	cat << "EOF" > $startdir/pkg/sbin/upgradepkg
#!/bin/sh
/sbin/spkg -u "$@"
exit $?
EOF

	cat << "EOF" > $startdir/pkg/sbin/removepkg
#!/bin/sh
/sbin/spkg -d "$@"
exit $?
EOF

	# make wrapper scripts executable
	chmod 755 $startdir/pkg/sbin/*

	#
	### pkgtools section
	#

	cd $startdir/src/pkgtools

	# Install Slackware script manpages:
	cd manpages
	mkdir -p $startdir/pkg/usr/man/man8
	for page in installpkg.8 upgradepkg.8 removepkg.8 ; do
		sed -i 's#installpkg#slack-installpkg#' $page
		sed -i 's#upgradepkg#slack-upgradepkg#' $page
		sed -i 's#removepkg#slack-removepkg#' $page
		cat $page | gzip -9c > $startdir/pkg/usr/man/man8/slack-$page.gz
	done
	for page in explodepkg.8 makepkg.8 pkgtool.8 ; do
		sed -i 's#installpkg#slack-installpkg#' $page
		sed -i 's#upgradepkg#slack-upgradepkg#' $page
		sed -i 's#removepkg#slack-removepkg#' $page
		cat $page | gzip -9c > $startdir/pkg/usr/man/man8/$page.gz
	done
	cd ..

	# Install Slackware scripts:
	cd scripts

	# Install the core Slackware package tools:
	mkdir -p $startdir/pkg/sbin
	for file in installpkg removepkg upgradepkg ; do
		sed -i 's#installpkg#slack-installpkg#' $file
		sed -i 's#upgradepkg#slack-upgradepkg#' $file
		sed -i 's#removepkg#slack-removepkg#' $file
		cp -a $file $startdir/pkg/sbin/slack-$file
	done
	for file in explodepkg makebootdisk makepkg pkgtool ; do
		sed -i 's#installpkg#slack-installpkg#' $file
		sed -i 's#upgradepkg#slack-upgradepkg#' $file
		sed -i 's#removepkg#slack-removepkg#' $file
		cp -a $file $startdir/pkg/sbin
	done
	chown root:root $startdir/pkg/sbin/*
	chmod 755 $startdir/pkg/sbin/*

	# These scripts are used during the installation:
	mkdir -p $startdir/pkg/var/log/setup/tmp
	chmod 700 $startdir/pkg/var/log/setup/tmp
	for file in setup.* ; do
		sed -i 's#installpkg#slack-installpkg#' $file
		sed -i 's#upgradepkg#slack-upgradepkg#' $file
		sed -i 's#removepkg#slack-removepkg#' $file
		cp -a $file $startdir/pkg/var/log/setup
	done

	# use our own setup.services script
	cd $startdir/src/
	mv setup.services $startdir/pkg/var/log/setup/ || return 1

	# chown & chmod
	chown root:root $startdir/pkg/var/log/setup/setup.*
	chmod 755 $startdir/pkg/var/log/setup/setup.*
}

create_slackdesc() {
mkdir $startdir/pkg/install
cat <<"EODESC" >$startdir/pkg/install/slack-desc
spkg-pkgtools: spkg-pkgtools - package manager
spkg-pkgtools: 
spkg-pkgtools: spkg is implemented in C and optimized for speed. This package
spkg-pkgtools: contains wrapper script for original Slackware tools installpkg,
spkg-pkgtools: upgradepkg and removepkg.
spkg-pkgtools: 
spkg-pkgtools: 
spkg-pkgtools: 
spkg-pkgtools: 
spkg-pkgtools: 
spkg-pkgtools: 
EODESC
}

create_source_file(){
	[ -f $package.src ] && rm $package.src
	if [ ! -z $sourcetemplate ]; then
		echo $sourcetemplate/SLKBUILD >> $package.src
		echo $sourcetemplate/build-$pkgname.sh >> $package.src
		for SOURCES in ${source[@]}; do
			protocol=$(echo $SOURCES | sed 's|:.*||')
			if ! [ "$protocol" = "http" -o "$protocol" = "https" -o "$protocol" = "ftp" ]; then
				if [ ! -z $sourcetemplate ]; then
					echo $sourcetemplate/$(basename $SOURCES) >> $package.src
				else
					echo $(basename $SOURCES) >> $package.src
				fi
			else
				echo $SOURCES >> $package.src
			fi
		done
	fi
}
post_checks(){
	# Ideas taken from src2pkg :)
	if [ -d "$startdir/pkg/usr/doc/$pkgname-$pkgver" ]; then
		for DIRS in usr/doc/$pkgname-$pkgver usr/doc; do
			cd $startdir/pkg/$DIRS
			if [[ $(find . -type f) = "" ]] ; then
				cd ..
				rmdir $DIRS
			fi
		done
	fi
	# if the docs weren't deleted ...
	if [ -d "$startdir/pkg/usr/doc/$pkgname-$pkgver" ]; then
		cd $startdir/pkg/usr/doc/$pkgname-$pkgver
		#remove zero lenght files
		if [[ $(find . -type f -size 0) ]]; then
			echo "Removing some zero lenght files"
			find . -type f -size 0 -exec rm -f {} \;
		fi
	fi
	# check if we need to add code to handle info pages
	if [[ -d $startdir/pkg/usr/info ]] && [[ ! $(grep install-info $startdir/pkg/install/doinst.sh &> /dev/null) ]] ; then
		echo "Found info files - Adding install-info command to doinst.sh"
		INFO_LIST=$(ls -1 $startdir/pkg/usr/info)
		echo "" >> $startdir/pkg/install/doinst.sh
		echo "if [ -x usr/bin/install-info ] ; then" >> $startdir/pkg/install/doinst.sh
		for page in $(echo $INFO_LIST) ; do
			echo " usr/bin/install-info --info-dir=usr/info usr/info/$page 2>/dev/null" >> $startdir/pkg/install/doinst.sh
		done
		echo "fi" >> $startdir/pkg/install/doinst.sh
	fi
	[[ -e $startdir/pkg/usr/info/dir ]] && rm -f $startdir/pkg/usr/info/dir

	if [ -d $startdir/pkg/etc ]; then
		cd $startdir/pkg/
		for conf in $(find ./etc -type f) ; do
			conf=${conf: 2}
			dotnew=( "${dotnew[@]}" "$conf" )
		done
	fi
	if [[ "$dotnew" ]]; then
        for files in ${dotnew[@]} ; do
                fullfile="${startdir}/pkg/${files}"
                if [ -e "$fullfile" ]; then
                        mv $fullfile ${fullfile}.new
                else
                        echo "$fullfile was not found"
                        exit 2
                fi
        done
        cat<<"EODOTNEW" >>$startdir/pkg/install/doinst.sh
#Added by slkbuild 0.5.1
dotnew() {
        NEW="${1}.new"
        OLD="$1"
        if [ ! -e $OLD ]; then
                mv $NEW $OLD
        elif [ "$(cat $OLD | md5sum)" = "$(cat $NEW | md5sum)" ]; then
                rm $NEW
        fi
}
EODOTNEW
for i in ${dotnew[@]}; do
echo "dotnew $i" >> $startdir/pkg/install/doinst.sh
done
fi
}

####End Non Redundant Code############################

#Variables

startdir=$(pwd)
SRC=$startdir/src
PKG=$startdir/pkg

pkgname=spkg-pkgtools
pkgver=1.0rc11
pkgrel=9tm
arch=i486
package=$pkgname-$pkgver-$arch-9tm
source=("http://spkg.megous.com/dl/releases/spkg-1.0rc11.tar.gz" "get-new-src.sh" "http://thenktor.dyndns.org/packages/spkg-pkgtools/pkgtools-13.0.src.tar.gz" "01-txz.patch" "02-size.patch" "http://thenktor.dyndns.org/packages/spkg-pkgtools/setup.services")
sourcetemplate=http://thenktor.dyndns.org/packages/spkg-pkgtools
docs=()
export CFLAGS="-O2 -march=i486 -mtune=i686"
export CXXFLAGS="-O2 -march=i486 -mtune=i686"

#Execution

check_for_root
clean_old_builds
prepare_directory
extract_source
set_pre_permissions
build
if [ ! "$?" = "0" ]; then
	echo "build() failed."
	exit 2
fi
create_slackdesc
post_checks
strip_binaries
gzip_man_and_info_pages
set_post_permissions
copy_build_script
create_package
create_source_file
echo "Package has been built."
echo "Cleaning pkg and src directories"
clean_dirs
